@@ -1,15 +1,12 @@
 test_that("Testing if splitting edges, and adding obsevations casuse error", {
 
-line1 <- Line(rbind(c(0,0),c(1,0)))
-line2 <- Line(rbind(c(0,0),c(0,1)))
-line3 <- Line(rbind(c(0,1),c(-1,1)))
+edge1 <- rbind(c(0,0),c(1,0))
+edge2 <- rbind(c(0,0),c(0,1))
+edge3 <- rbind(c(0,1),c(-1,1))
 theta <- seq(from=pi,to=3*pi/2,length.out = 20)
-line4 <- Line(cbind(sin(theta),1+ cos(theta)))
-Lines = sp::SpatialLines(list(Lines(list(line1),ID="1"),
-                              Lines(list(line2),ID="2"),
-                              Lines(list(line4),ID="3"),
-                              Lines(list(line3),ID="4")))
-graph <- metric_graph$new(lines = Lines)
+edge4 <- cbind(sin(theta),1+ cos(theta))
+edges = list(edge1, edge2, edge3, edge4)
+graph <- metric_graph$new(edges = edges)
 
 graph$.__enclos_env__$private$split_edge(1,0.5)
 
@@ -24,8 +21,8 @@ graph$plot(data="y")
 
 xc = c(-0.15,-0.51, 0.35,0.45)
 yc = c(0.4, 0.25,0,0.4)
-Spoints = SpatialPoints(cbind(xc, yc))
-Spoints = SpatialPointsDataFrame(Spoints,  data.frame(a=1:4))
+Spoints = sp::SpatialPoints(cbind(xc, yc))
+Spoints = sp::SpatialPointsDataFrame(Spoints,  data.frame(a=1:4))
 
 df_2 <- data.frame(y = c(1,2,3,4))
 graph$add_observations(Spoints, data = df_2)
